#ifdef _VARIANT_ARDUINO_STM32_
	#define dWF(a,b) digitalWriteFast(digitalPinToPinName(a),b)
	#define dRF(a) digitalReadFast(digitalPinToPinName(a))
	#define HW_TIMER
	#ifdef HW_TIMER
		bool bLed=0;	//builtin led state
		uint8_t bLedCount=0;
		bool blinkLED=0;
    uint16_t counter=0;
    uint16_t tick=0;
	#endif	
#else
	#include "fastdio.h"
	#define dWF(a,b) digitalWriteFast(a,b)
	#define dRF(a) digitalReadFast(a)
#endif


#define PIN0w  PA3
#define PIN1w  PA4

#define PIN2r  PA1
#define PIN3r  PB1

#define CYCLES 100000

#define LED LED_BUILTIN

void setup() {
  // put your setup code here, to run once:
	pinMode(LED,OUTPUT);
  pinMode(PIN0w,OUTPUT);
  pinMode(PIN1w,OUTPUT);
  pinMode(PIN2r,INPUT);
  pinMode(PIN3r,INPUT);
  
	dWF(LED,0);
	Serial.begin(250000);
	
	
#ifdef HW_TIMER
	blinkLED=1;
	HardwareTimer *MyTimer =  new HardwareTimer(TIM1);
	MyTimer->setMode(2, TIMER_OUTPUT_COMPARE);
	MyTimer->setOverflow(100, HERTZ_FORMAT); // 100 Hz
	MyTimer->attachInterrupt(TimerUpd);
	MyTimer->resume();
#endif
  while(!Serial){}
}

void loop() {
  uint32_t ts,te;
  Serial.println("********************************************************************************************");
  Serial.println("Port I/O speed test. Arduino digitalRead/Write / digitalReadFast/WriteFast / STM32 register ");
  
//***********************************************  WRITE tests ******************************************//
  Serial.println("********* Write test: *********");
  Serial.print(" digitalWrite: ");
  ts=micros();
  for(uint32_t i=0;i<CYCLES;i++)
  {
    digitalWrite(PIN0w,1);
    digitalWrite(PIN0w,0);
    digitalWrite(PIN0w,1);
    digitalWrite(PIN0w,0);
    digitalWrite(PIN0w,1);
    digitalWrite(PIN0w,0);
    digitalWrite(PIN0w,1);
    digitalWrite(PIN0w,0);
    digitalWrite(PIN0w,1);
    digitalWrite(PIN0w,0);
    digitalWrite(PIN0w,1);
    digitalWrite(PIN0w,0);
    digitalWrite(PIN0w,1);
    digitalWrite(PIN0w,0);
    digitalWrite(PIN0w,1);
    digitalWrite(PIN0w,0);
    digitalWrite(PIN0w,1);
    digitalWrite(PIN0w,0);
    digitalWrite(PIN0w,1);
    digitalWrite(PIN0w,0);    
  }
  te=micros();
  float dW_time = (float)(te-ts)/(CYCLES*20);
  Serial.print(te-ts);
  Serial.print("us    1op time: ");
  Serial.print(dW_time);
  Serial.print("   op/s: ");
  Serial.print((float)1/dW_time);
  Serial.println("M/s");


  Serial.print(" digitalWriteFast: ");
  ts=micros();
  for(uint32_t i=0;i<CYCLES;i++)
  {
    dWF(PIN0w,1);
    dWF(PIN0w,0);
    dWF(PIN0w,1);
    dWF(PIN0w,0);
    dWF(PIN0w,1);
    dWF(PIN0w,0);
    dWF(PIN0w,1);
    dWF(PIN0w,0);
    dWF(PIN0w,1);
    dWF(PIN0w,0);
    dWF(PIN0w,1);
    dWF(PIN0w,0);
    dWF(PIN0w,1);
    dWF(PIN0w,0);
    dWF(PIN0w,1);
    dWF(PIN0w,0);
    dWF(PIN0w,1);
    dWF(PIN0w,0);
    dWF(PIN0w,1);
    dWF(PIN0w,0);    
  }
  te=micros();
  float dWF_time = (float)(te-ts)/(CYCLES*20);
  Serial.print(te-ts);
  Serial.print("us    1op time: ");
  Serial.print(dWF_time);
  Serial.print("   op/s: ");
  Serial.print((float)1/dWF_time);
  Serial.println("M/s");

  
#ifdef _VARIANT_ARDUINO_STM32_
  Serial.print(" GPIOx->ODR: ");
  ts=micros();
  for(uint32_t i=0;i<CYCLES;i++)
  {
    //PA3
    GPIOA->ODR &= 0b1111111111110111; //0xFFF7
    GPIOA->ODR |= 0b0000000000001000; //0x0008
    GPIOA->ODR &= 0b1111111111110111; 
    GPIOA->ODR |= 0b0000000000001000; 
    GPIOA->ODR &= 0b1111111111110111; 
    GPIOA->ODR |= 0b0000000000001000; 
    GPIOA->ODR &= 0b1111111111110111; 
    GPIOA->ODR |= 0b0000000000001000; 
    GPIOA->ODR &= 0b1111111111110111; 
    GPIOA->ODR |= 0b0000000000001000; 
    GPIOA->ODR &= 0b1111111111110111; 
    GPIOA->ODR |= 0b0000000000001000; 
    GPIOA->ODR &= 0b1111111111110111; 
    GPIOA->ODR |= 0b0000000000001000; 
    GPIOA->ODR &= 0b1111111111110111; 
    GPIOA->ODR |= 0b0000000000001000; 
    GPIOA->ODR &= 0b1111111111110111; 
    GPIOA->ODR |= 0b0000000000001000; 
    GPIOA->ODR &= 0b1111111111110111; 
    GPIOA->ODR |= 0b0000000000001000; 
  }
  te=micros();
  float dGPo_time = (float)(te-ts)/(CYCLES*20);
  Serial.print(te-ts);
  Serial.print("us    1op time: ");
  Serial.print(dGPo_time);
  Serial.print("   op/s: ");
  Serial.print((float)1/dGPo_time);
  Serial.println("M/s");
#endif

//***********************************************  READ tests ******************************************//
  Serial.println("********* Read test: *********");
  Serial.print(" digitalRead: ");
  ts=micros();
  bool aa;
  for(uint32_t i=0;i<CYCLES;i++)
  {
    aa = digitalRead(PIN2r);
    aa = digitalRead(PIN3r);
    aa = digitalRead(PIN2r);
    aa = digitalRead(PIN3r);
    aa = digitalRead(PIN2r);
    aa = digitalRead(PIN3r);
    aa = digitalRead(PIN2r);
    aa = digitalRead(PIN3r);
    aa = digitalRead(PIN2r);
    aa = digitalRead(PIN3r);
    aa = digitalRead(PIN2r);
    aa = digitalRead(PIN3r);
    aa = digitalRead(PIN2r);
    aa = digitalRead(PIN3r);
    aa = digitalRead(PIN2r);
    aa = digitalRead(PIN3r);
    aa = digitalRead(PIN2r);
    aa = digitalRead(PIN3r);
    aa = digitalRead(PIN2r);
    aa = digitalRead(PIN3r);
  }
  te=micros();
  Serial.print(te-ts);
  Serial.print("us    1op time: ");
  float dR_time = (float)(te-ts)/(CYCLES*20);
  Serial.print(dR_time);
  Serial.print(" ops/s: ");
  Serial.print((float)1/dR_time);
  Serial.println(" M/s");

  Serial.print(" digitalReadFast: ");
  ts=micros();
  for(uint32_t i=0;i<CYCLES;i++)
  {
    aa = dRF(PIN2r);
    aa = dRF(PIN3r);
    aa = dRF(PIN2r);
    aa = dRF(PIN3r);
    aa = dRF(PIN2r);
    aa = dRF(PIN3r);
    aa = dRF(PIN2r);
    aa = dRF(PIN3r);
    aa = dRF(PIN2r);
    aa = dRF(PIN3r);
    aa = dRF(PIN2r);
    aa = dRF(PIN3r);
    aa = dRF(PIN2r);
    aa = dRF(PIN3r);
    aa = dRF(PIN2r);
    aa = dRF(PIN3r);
    aa = dRF(PIN2r);
    aa = dRF(PIN3r);
    aa = dRF(PIN2r);
    aa = dRF(PIN3r);
  }
  te=micros();
  float dRF_time = (float)(te-ts)/(CYCLES*20);
  Serial.print(te-ts);
  Serial.print("us    1op time: ");
  Serial.print(dRF_time);
  Serial.print(" ops/s: ");
  Serial.print((float)1/dRF_time);
  Serial.println(" M/s");

#ifdef _VARIANT_ARDUINO_STM32_
  Serial.print(" GPIOx->IDR: ");
  ts=micros();
  for(uint32_t i=0;i<CYCLES;i++)
  {
    aa = GPIOA->IDR & 0b0000000000000010;
    aa = GPIOB->IDR & 0b0000000000000010;
    aa = GPIOA->IDR & 0b0000000000000010;
    aa = GPIOB->IDR & 0b0000000000000010;
    aa = GPIOA->IDR & 0b0000000000000010;
    aa = GPIOB->IDR & 0b0000000000000010;
    aa = GPIOA->IDR & 0b0000000000000010;
    aa = GPIOB->IDR & 0b0000000000000010;
    aa = GPIOA->IDR & 0b0000000000000010;
    aa = GPIOB->IDR & 0b0000000000000010;
    aa = GPIOA->IDR & 0b0000000000000010;
    aa = GPIOB->IDR & 0b0000000000000010;
    aa = GPIOA->IDR & 0b0000000000000010;
    aa = GPIOB->IDR & 0b0000000000000010;
    aa = GPIOA->IDR & 0b0000000000000010;
    aa = GPIOB->IDR & 0b0000000000000010;
    aa = GPIOA->IDR & 0b0000000000000010;
    aa = GPIOB->IDR & 0b0000000000000010;
    aa = GPIOA->IDR & 0b0000000000000010;
    aa = GPIOB->IDR & 0b0000000000000010;
  }
  te=micros();
  float dGPi_time = (float)(te-ts)/(CYCLES*20);
  Serial.print(te-ts);
  Serial.print("us    1op time: ");
  Serial.print(dGPi_time);
  Serial.print(" ops/s: ");
  Serial.print((float)1/dGPi_time);
  Serial.println(" M/s");
#endif

Serial.println();
Serial.println();
Serial.println();
Serial.println();

  delay(10000);
}


#ifdef HW_TIMER
void TimerUpd()
{
	//Blink LED 10Hz
	if(blinkLED)
	bLedCount++;
  if(bLedCount==1)
  {
    bLed=1;
    dWF(LED,bLed);
  }
	else if(bLedCount==50)
	{
		bLed=0;
		bLedCount=0;
    dWF(LED,bLed);
	}	
}
#endif
